@@ -13,6 +13,7 @@
 #include <algorithm>
 #include <numeric>
 #include <limits>
+#include <deque>
 
 #include "ModelSpecifics.h"
 #include "Iterators.h"
@@ -26,10 +27,6 @@
 #include <tbb/parallel_for.h>
 #endif
 
-
-//#include "R.h"
-//#include "Rcpp.h" // TODO Remove
-
 #ifdef CYCLOPS_DEBUG_TIMING
 	#include "Timing.h"
 #endif
@@ -769,49 +766,69 @@ void ModelSpecifics<BaseModel,RealType>::getSchoenfeldResidualsImpl(int index,
     RealType wHessian = static_cast<RealType>(0);
     RealType xHessian = static_cast<RealType>(0);
 
+    RealType uGradient2 = static_cast<RealType>(0);
+    RealType wGradient2 = static_cast<RealType>(0);
+
     // TODO: only written for accummulive models (Cox, Fine/Grey)
 
 	IteratorType it(hX, index);
 
 	RealType resNumerator  = static_cast<RealType>(0);
-	RealType resDenominator = static_cast<RealType>(0);
+	// RealType resDenominator = static_cast<RealType>(0);
 	RealType scoreNumerator1 = static_cast<RealType>(0);
 	RealType scoreNumerator2 = static_cast<RealType>(0);
 	RealType scoreDenominator = static_cast<RealType>(0);
 
 	// find start relavent accumulator reset point
 	auto reset = begin(accReset);
-	while( *reset < it.index() ) {
-		++reset;
-	}
+	// while( *reset < it.index() ) {
+	// 	++reset;
+	// }
+
+	std::deque<RealType> deferredResidualX;
+	std::deque<RealType> deferredScore1;
+	std::deque<RealType> deferredScore2;
+	std::deque<RealType> deferredScore3;
+	std::deque<int> deferredScore4;
 
 	auto processRow = [&](int i, RealType x) {
 
-	    // std::cerr << "row " << i;
+	    if (*reset <= hPid[i]) {
 
-		if (*reset <= i) {
+	        resNumerator = static_cast<RealType>(0);
+	        // resDenominator = static_cast<RealType>(0);
+	        scoreNumerator1 = static_cast<RealType>(0);
+	        scoreNumerator2 = static_cast<RealType>(0);
+	        scoreDenominator = static_cast<RealType>(0);
 
-		    // std::cerr << " reset ";
-
-			resNumerator = static_cast<RealType>(0);
-			resDenominator = static_cast<RealType>(0);
-			scoreNumerator1 = static_cast<RealType>(0);
-			scoreNumerator2 = static_cast<RealType>(0);
-			scoreDenominator = static_cast<RealType>(0);
-
-			++reset;
-		}
+	        ++reset;
+	    }
 
 		const auto expXBeta = offsExpXBeta[i]; // std::exp(hXBeta[i]);
 
-		resNumerator += expXBeta * x;
-		resDenominator += expXBeta;
+		resNumerator += hKWeight[i] * expXBeta * x;
+		// resDenominator += expXBeta;
 
-		if (hY[i] == 1 ) {
-		    // std::cerr << " " << x << " for " << resNumerator << " / " << resDenominator;
+		if (hY[i] == 1) {
+		    // std::cerr << " " << x << " for " << resNumerator << " / " << resDenominator << " - " <<  ( x - resNumerator / resDenominator) << "\n";
+		    // std::cerr << denomPid[i] << "\n";
 			if (hasResiduals) {
-			    const auto residual = x - resNumerator / resDenominator;
-				residuals->push_back(residual);
+			    if (i < (K - 1) && hPid[i] < *reset &&
+                    (hOffs[i] == hOffs[i+ 1]) && hY[i + 1] == 1) {
+			        deferredResidualX.push_back(x);
+			    } else {
+
+			        const auto denom = accDenomPid[hPid[i]];
+
+			        while (deferredResidualX.size() > 0) {
+			            const auto residual = deferredResidualX.front() - resNumerator / denom;
+			            residuals->push_back(residual);
+			            deferredResidualX.pop_front();
+			        }
+
+			        const auto residual = x - resNumerator / denom;
+				    residuals->push_back(residual);
+			    }
 			}
 			if (hasTimes) {
 				times->push_back(hOffs[i]);
@@ -819,128 +836,64 @@ void ModelSpecifics<BaseModel,RealType>::getSchoenfeldResidualsImpl(int index,
 			if (hasStrata) {
 				strata->push_back(hPidOriginal[i]);
 			}
+		} else {
+		    // std::cerr << "\n";
 		}
 
 		if (hasScore) {
-			const auto weight = covariate[i];
-			// MAS does not believe reweighing is correct, but is matching cox.zph
-
-			// const auto xt = x * covariate[i];
-			// MAS believes covariate should be adjusted
-			const auto numerator1 = expXBeta * x; // TODO not xt?
-			const auto numerator2 = expXBeta * x * x; // TODO not xt?
+			const auto cov = covariate[i];
+			const auto numerator1 = hKWeight[i] * expXBeta * x;
+			const auto numerator2 = hKWeight[i] * expXBeta * x * x;
 
 			if (hY[i] == 1) {
-				uGradient += x;
-				wGradient += x * weight; // TODO not xt and no weight?
+				uGradient2 += hKWeight[i] * x;
+				wGradient2 += hKWeight[i] * x * cov;
 			}
 
 			scoreNumerator1 += numerator1;
 			scoreNumerator2 += numerator2;
 
-			const auto t = scoreNumerator1 / resDenominator;
-			const auto gradient = hNWeight[i] * t;
-			const auto hessian = hNWeight[i] * (scoreNumerator2 / resDenominator - t * t);
+		    if (i < (K - 1) && hPid[i] == hPid[i + 1]) {
+                // zero contribution under Breslow
+		    } else {
 
-			uGradient -= gradient;
-			wGradient -= gradient * weight;
+			    const auto denom = accDenomPid[hPid[i]];
 
-			uHessian += hessian;
-			wHessian += hessian * weight * weight;
-			xHessian += hessian * weight;
+	    		const auto t = scoreNumerator1 / denom;
+			    const auto gradient = hNWeight[hPid[i]] * t;
+			    const auto hessian = hNWeight[hPid[i]] * (scoreNumerator2 / denom - t * t);
+
+			    uGradient += gradient;
+			    wGradient += gradient * cov;
+
+			    // std::cerr << "newA" << i << " " << hPid[i] << " " << scoreNumerator1 << "/" << denom << " " << hY[i] << " " << uGradient << " " << hNWeight[hPid[i]] << "\n";
+
+			    uHessian += hessian;
+			    wHessian += hessian * cov * cov;
+			    xHessian += hessian * cov;
+		    }
 		}
-
-		// std::cerr << "\n";
 	};
 
 	// main loop
-	for (; it; ) {
-
-		int i = it.index();
-		const RealType x = it.value();
-
-		processRow(i, x);
-
-		++it;
-
-		if (IteratorType::isSparse) {
-
-			const int next = it ? it.index() : N;
-			for (++i; i < next; ++i) {
-
-				processRow(i, static_cast<RealType>(0));
-
-// 				if (*reset <= i) {
-// 					resNumerator = static_cast<RealType>(0);
-// 					resDenominator = static_cast<RealType>(0);
-//
-// 					scoreNumerator1 = static_cast<RealType>(0);
-// 					scoreNumerator2 = static_cast<RealType>(0);
-// 					scoreDenominator = static_cast<RealType>(0);
-//
-// 					++reset;
-// 				}
-//
-// 				const auto expXBeta = offsExpXBeta[i]; // std::exp(hXBeta[i]);
-//
-// 				resDenominator += expXBeta;
-//
-// 				if (hY[i] == 1) {
-// 					if (hasResiduals) {
-// 						residuals->push_back(-resNumerator / resDenominator);
-// 					}
-// 					if (hasTimes) {
-// 						times->push_back(hOffs[i]);
-// 					}
-// 					if (hasStrata) {
-// 						strata->push_back(hPidOriginal[i]);
-// 					}
-// 				}
-//
-// 				if (hasScore) {
-// 					const auto weight = covariate[i];
-// 					// MAS does not believe reweighing is correct, but is matching cox.zph
-//
-// 					// const auto xt = x * covariate[i];
-// 					// MAS believes covariate should be adjusted
-// 					const auto numerator1 = expXBeta * x; // TODO not xt?
-// 					const auto numerator2 = expXBeta * x * x; // TODO not xt?
-//
-// 					if (hY[i] == 1) {
-// 						uGradient += x;
-// 						wGradient += x * weight; // TODO not xt and no weight?
-// 					}
-//
-// 					scoreNumerator1 += numerator1;
-// 					scoreNumerator2 += numerator2;
-//
-// 					const auto t = scoreNumerator1 / resDenominator;
-// 					const auto gradient = hNWeight[i] * t;
-// 					const auto hessian = hNWeight[i] * (scoreNumerator2 / resDenominator - t * t);
-//
-// 					uGradient -= gradient;
-// 					wGradient -= gradient * weight;
-//
-// 					uHessian += hessian;
-// 					wHessian += hessian * weight * weight;
-// 				}
-			}
-		}
+	for (int i = 0; i < K; ++i) {
+	    if (i == it.index()) {
+	        processRow(i, it.value());
+	        ++it;
+	    } else {
+	        processRow(i, static_cast<RealType>(0));
+	    }
 	}
 
     if (hasScore) {
 
-        score[0] = static_cast<double>(uGradient);
-        score[1] = static_cast<double>(wGradient);
+        score[0] = static_cast<double>(uGradient2 - uGradient);
+        score[1] = static_cast<double>(wGradient2 - wGradient);
         score[2] = static_cast<double>(uHessian);
         score[3] = static_cast<double>(xHessian);
         score[4] = static_cast<double>(xHessian);
         score[5] = static_cast<double>(wHessian);
-
-        // *score = static_cast<double>(gradient / hessian);
-        // std::cerr << "score = " << wGradient << " / " << wHessian << "\n";
-        // std::cerr << "hess2 = " << hess2 << " or " <<  static_cast<RealType>(2.0) * hXjX[index] << "\n";
-    }
+     }
 }
 
 // TODO The following function is an example of a double-dispatch, rewrite without need for virtual function
@@ -1180,10 +1133,14 @@ void ModelSpecifics<BaseModel,RealType>::computeGradientAndHessianImpl(int index
                         w, // Signature-only, for iterator-type specialization
                         &gradient, &hessian, accNumerPid, accNumerPid2,
                         accDenomPid[i], hNWeight[i], 0.0, hXBeta[i], hY[i]); // When function is in-lined, compiler will only use necessary arguments
+
+                // std::cerr << "oldA" << i << " " << accNumerPid << "/" << accDenomPid[i] << " " << hY[i] << " " << gradient << " " << hNWeight[i] << "\n";
+
                 ++it;
 
     	        if (IteratorType::isSparse) {
 
+    	            // TODO Should end be N or K?
     	            const int next = it ? it.index() : N;
                     for (++i; i < next; ++i) {
 
@@ -1197,6 +1154,8 @@ void ModelSpecifics<BaseModel,RealType>::computeGradientAndHessianImpl(int index
                                 w, // Signature-only, for iterator-type specialization
     	                        &gradient, &hessian, accNumerPid, accNumerPid2,
     	                        accDenomPid[i], hNWeight[i], static_cast<RealType>(0), hXBeta[i], hY[i]); // When function is in-lined, compiler will only use necessary arguments
+
+                        // std::cerr << "oldB" << i << " " << accNumerPid << "/" << accDenomPid[i] << " " << hY[i] << " " << gradient << " " << hNWeight[i] << "\n";
     	            }
     	        }
     	    }
@@ -2288,6 +2247,7 @@ void ModelSpecifics<BaseModel,RealType>::setPidForAccumulationImpl(const AnyReal
 
     // Find first non-zero weight
     size_t index = 0;
+
     while(weights != nullptr && weights[index] == 0.0 && index < K) {
         hPid[index] = ignore;
         index++;
@@ -2299,6 +2259,8 @@ void ModelSpecifics<BaseModel,RealType>::setPidForAccumulationImpl(const AnyReal
 
     int pid = hPid[index] = 0;
 
+    int nTies = 0;
+
     for (size_t k = index + 1; k < K; ++k) {
         if (weights == nullptr || weights[k] != 0.0) {
             int nextPid = hPid[k];
@@ -2306,11 +2268,13 @@ void ModelSpecifics<BaseModel,RealType>::setPidForAccumulationImpl(const AnyReal
             if (nextPid != lastPid) { // start new strata
                 pid++;
                 accReset.push_back(pid);
+                nTies = 0;
                 lastPid = nextPid;
             } else {
 
                 if (lastEvent == 1.0 && lastTime == hOffs[k] && lastEvent == hY[k]) {
                     // In a tie, do not increment denominator
+                    ++nTies;
                 } else {
                     pid++;
                 }
